﻿# -*- coding: utf-8 -*-
#######################################################################################################
#
#    MediaPortal for Dreambox OS
#
#    Coded by MediaPortal Team (c) 2013-2021
#
#  This plugin is open source but it is NOT free software.
#
#  This plugin may only be distributed to and executed on hardware which
#  is licensed by Dream Property GmbH. This includes commercial distribution.
#  In other words:
#  It's NOT allowed to distribute any parts of this plugin or its source code in ANY way
#  to hardware which is NOT licensed by Dream Property GmbH.
#  It's NOT allowed to execute this plugin and its source code or even parts of it in ANY way
#  on hardware which is NOT licensed by Dream Property GmbH.
#
#  This applies to the source code as a whole as well as to parts of it, unless explicitely
#  stated otherwise.
#
#  If you want to use or modify the code or parts of it, permission from the authors is necessary.
#  You have to keep OUR license and inform us about any modification, but it may NOT be distributed
#  other than under the conditions noted above.
#
#  As an exception regarding modifcations, you are NOT permitted to remove
#  any copy protections implemented in this plugin or change them for means of disabling
#  or working around the copy protections, unless the change has been explicitly permitted
#  by the original authors. Also decompiling and modification of the closed source
#  parts is NOT permitted.
#
#  Advertising with this plugin is NOT allowed.
#
#  For other uses, permission from the authors is necessary.
#
#######################################################################################################

from future import standard_library
standard_library.install_aliases()
from builtins import map
from ...plugin import _
from ...resources.imports import *
from ...resources.twagenthelper import TwAgentHelper

agent='Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/73.0.3683.86 Safari/537.36'
default_cover = "file://%s/freeones.png" % (config_mp.mediaportal.iconcachepath.value + "logos")

class freeonesGenreScreen(MPScreen):

	def __init__(self, session):
		MPScreen.__init__(self, session, skin='MP_Plugin', default_cover=default_cover)

		self["actions"] = ActionMap(["MP_Actions"], {
			"ok" : self.keyOK,
			"0" : self.closeAll,
			"cancel" : self.keyCancel,
			"up" : self.keyUp,
			"down" : self.keyDown,
			"right" : self.keyRight,
			"left" : self.keyLeft
		}, -1)

		self['title'] = Label("FreeOnes.com")
		self['ContentTitle'] = Label("Genre:")

		self.keyLocked = True
		self.suchString = ''

		self._items = []
		self.ml = MenuList([], enableWrapAround=True, content=eListboxPythonMultiContent)
		self['liste'] = self.ml

		self.onLayoutFinish.append(self.layoutFinished)

	def layoutFinished(self):
		self.keyLocked = True
		url = "https://www.freeones.com/categories"
		twAgentGetPage(url, agent=agent).addCallback(self.genreData).addErrback(self.dataError)

	def genreData(self, data):
		parse = re.search('teaser-mode--grid(.*?)<footer', data, re.S)
		Cats = MPfindall(self, '"teaser-category".*?href=".*?".*?<img.*?src="(.*?)".*?alt="(.*?)"', parse.group(1), re.S)
		if Cats:
			for (Image, Title) in Cats:
				Title = Title.strip()
				self._items.append((Title, Title.replace(' ', '%20'), Image))
		self._items.sort()
		self._items.insert(0, ("Longest", "https://www.freeones.com/videos?f[contentType]=video&s=duration&o=desc", default_cover))
		self._items.insert(0, ("Most Viewed", "https://www.freeones.com/videos?f[contentType]=video&s=views&o=desc", default_cover))
		self._items.insert(0, ("Top Rated", "https://www.freeones.com/videos?f[contentType]=video&s=votes.average&o=desc", default_cover))
		self._items.insert(0, ("Latest", "https://www.freeones.com/videos?f[contentType]=video&s=latest&o=desc", default_cover))
		self._items.insert(0, ("--- Search ---", "callSuchen", default_cover))
		self.ml.setList(list(map(self._defaultlistcenter, self._items)))
		self.keyLocked = False
		self.showInfos()

	def showInfos(self):
		Image = self['liste'].getCurrent()[0][2]
		CoverHelper(self['coverArt']).getCover(Image, agent=agent)

	def keyOK(self):
		if self.keyLocked:
			return
		Name = self['liste'].getCurrent()[0][0]
		if Name == "--- Search ---":
			self.suchen()
		else:
			Link = self['liste'].getCurrent()[0][1]
			self.session.open(freeonesFilmScreen, Link, Name)

	def SuchenCallback(self, callback = None):
		if callback is not None and len(callback):
			Name = "--- Search ---"
			self.suchString = callback
			Link = urllib.parse.quote(callback).replace(' ', '%20')
			self.session.open(freeonesFilmScreen, Link, Name)

class freeonesFilmScreen(MPScreen):

	def __init__(self, session, Link, Name):
		self.Link = Link
		self.Name = Name
		MPScreen.__init__(self, session, skin='MP_Plugin', default_cover=default_cover)

		self["actions"] = ActionMap(["MP_Actions"], {
			"ok" : self.keyOK,
			"0" : self.closeAll,
			"cancel" : self.keyCancel,
			"up" : self.keyUp,
			"down" : self.keyDown,
			"right" : self.keyRight,
			"left" : self.keyLeft,
			"nextBouquet" : self.keyPageUp,
			"prevBouquet" : self.keyPageDown,
			"green" : self.keyPageNumber
		}, -1)

		self['title'] = Label("FreeOnes.com")
		self['ContentTitle'] = Label("Genre: %s" % self.Name)
		self['F2'] = Label(_("Page"))

		self['Page'] = Label(_("Page:"))
		self.keyLocked = True
		self.page = 1
		self.lastpage = 1

		self._items = []
		self.ml = MenuList([], enableWrapAround=True, content=eListboxPythonMultiContent)
		self['liste'] = self.ml

		self.onLayoutFinish.append(self.loadPage)

	def loadPage(self):
		self.keyLocked = True
		self['name'].setText(_('Please wait...'))
		self._items = []
		if re.match(".*?Search", self.Name):
			url = "https://www.freeones.com/videos?q=%s&v=teasers&s=relevance&o=desc&f[contentType]=video&p=%s" % (self.Link, str(self.page))
		else:
			if self.Link.startswith('http'):
				url = self.Link + "&p=" + str(self.page)
			else:
				url = "https://www.freeones.com/videos?q=&f[contentType]=video&f[categories]=%s&p=%s" % (self.Link, str(self.page))
		twAgentGetPage(url, agent=agent).addCallback(self.loadData).addErrback(self.dataError)

	def loadData(self, data):
		self.getLastPage(data, '<page-selector(.*?)>', '.*:max="(\d+)"')
		parse = re.search('class="mb-2 mb-md-3(.*?)</html>', data, re.S)
		Movies = MPfindall(self, 'class="grid-item.*?href="(.*?)".*?class="image-wrap(.*?)d-flex.*?label="duration.*?>(.*?)</span.*?"video-name" title="(.*?)".*?popularity.*?title="(.*?)">.*?views</p>', parse.group(1), re.S)
		if Movies:
			for (Url, MetaData, Runtime, BackTitle, Views) in Movies:
				Url = "https://www.freeones.com" + Url
				Views = Views.replace(',', '')
				Meta = MPfindall(self, '<img.*?src="(.*?)".*?alt="(.*?)"', MetaData, re.S)
				if Meta:
					Image = Meta[0][0]
					Title = Meta[0][1]
				else:
					Meta = MPfindall(self, '<img.*?alt="(.*?)".*?data-src="(.*?)"', MetaData, re.S)
					if Meta:
						Image = Meta[0][1]
						Title = Meta[0][0]
					else:
						Title = BackTitle.strip()
						Image = 'https://assets.freeones.com/themes/babe-light/placeholders/generic.svg'
				self._items.append((decodeHtml(Title), Url, Image, Runtime, Views))
		if len(self._items) == 0:
			self._items.append((_('No movies found!'), None, None, None, None))
		self._setList('_defaultlistleft', True)
		self.ml.moveToIndex(0)
		self.keyLocked = False
		self.showInfos()

	def showInfos(self):
		Url = self['liste'].getCurrent()[0][1]
		if Url == None:
			return
		title = self['liste'].getCurrent()[0][0]
		pic = self['liste'].getCurrent()[0][2]
		runtime = self['liste'].getCurrent()[0][3]
		views = self['liste'].getCurrent()[0][4]
		self['name'].setText(title)
		self['handlung'].setText("Runtime: %s\nViews: %s" % (runtime, views))
		CoverHelper(self['coverArt']).getCover(pic, agent=agent)

	def keyOK(self):
		if self.keyLocked:
			return
		Link = self['liste'].getCurrent()[0][1]
		if Link:
			twAgentGetPage(Link, agent=agent).addCallback(self.getVideoPage).addErrback(self.dataError)

	def getVideoPage(self, data):
		videoPage = MPfindall(self, 'streamingUrl:\s"(.*?)",', data, re.S)
		qualities = MPfindall(self, 'qualities:\s"(.*?)"', data, re.S)
		if videoPage and qualities:
			Title = self['liste'].getCurrent()[0][0]
			url = videoPage[0].replace('\/', '/').replace('\u003A', ':').replace('\u0020', '%20').replace('\u002D', '-') + "/" + qualities[0].split(',')[0] + ".mp4"
			mp_globals.player_agent = agent
			self.session.open(SimplePlayer, [(Title, url)], showPlaylist=False, ltype='freeones')